@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+   MA 02110-1301 USA */
 
 #include "comsat.h"
 #define obstack_chunk_alloc malloc
@@ -80,14 +81,14 @@ act_getline (FILE *fp, char **sptr, size_t *size)
 }
 
 static int
-expand_escape (char **pp, message_t msg, struct obstack *stk)
+expand_escape (char **pp, mu_message_t msg, struct obstack *stk)
 {
   char *p = *pp;
   char *start, *sval, *namep;
   int len;
-  header_t hdr;
-  body_t body;
-  stream_t stream;
+  mu_header_t hdr;
+  mu_body_t body;
+  mu_stream_t stream;
   int rc = 1;
   size_t size = 0, lncount = 0;
 
@@ -121,8 +122,8 @@ expand_escape (char **pp, message_t msg, struct obstack *stk)
 	break;
       memcpy (namep, start, len);
       namep[len] = 0;
-      if (message_get_header (msg, &hdr) == 0
-	  && header_aget_value (hdr, namep, &sval) == 0)
+      if (mu_message_get_header (msg, &hdr) == 0
+	  && mu_header_aget_value (hdr, namep, &sval) == 0)
 	{
 	  len = strlen (sval);
 	  obstack_grow (stk, sval, len);
@@ -146,15 +147,15 @@ expand_escape (char **pp, message_t msg, struct obstack *stk)
 	size = 400;
       if (lncount == 0)
 	lncount = maxlines;
-      if (message_get_body (msg, &body) == 0
-	  && body_get_stream (body, &stream) == 0)
+      if (mu_message_get_body (msg, &body) == 0
+	  && mu_body_get_stream (body, &stream) == 0)
 	{
 	  size_t nread;
 	  char *buf = malloc (size+1);
 
 	  if (!buf)
 	    break;
- 	  if (stream_read (stream, buf, size, 0, &nread) == 0)
+ 	  if (mu_stream_read (stream, buf, size, 0, &nread) == 0)
 	    {
 	      char *q;
 
@@ -180,7 +181,7 @@ expand_escape (char **pp, message_t msg, struct obstack *stk)
 }
 
 static char *
-expand_line (const char *str, message_t msg)
+expand_line (const char *str, mu_message_t msg)
 {
   const char *p;
   int c = 0;
@@ -197,7 +198,7 @@ expand_line (const char *str, message_t msg)
 	  p++;
 	  if (*p)
 	    {
-	      c = argcv_unescape_char (*p);
+	      c = mu_argcv_unquote_char (*p);
 	      obstack_1grow (&stk, c);
 	    }
 	  break;
@@ -330,7 +331,7 @@ open_rc (const char *filename, FILE *tty)
 }
 
 void
-run_user_action (FILE *tty, const char *cr, message_t msg)
+run_user_action (FILE *tty, const char *cr, mu_message_t msg)
 {
   FILE *fp;
   int nact = 0;
@@ -352,12 +353,12 @@ run_user_action (FILE *tty, const char *cr, message_t msg)
 	  str = expand_line (stmt, msg);
 	  if (!str)
 	    continue;
-	  if (argcv_get (str, "", NULL, &argc, &argv)
+	  if (mu_argcv_get (str, "", NULL, &argc, &argv)
 	      || argc == 0
 	      || argv[0][0] == '#')
 	    {
 	      free (str);
-	      argcv_free (argc, argv);
+	      mu_argcv_free (argc, argv);
 	      continue;
 	    }
 
