@@ -1,6 +1,6 @@
 ;;;; -*- scheme -*-
 ;;;; GNU Mailutils -- a suite of utilities for electronic mail
-;;;; Copyright (C) 2002, 2003, 2004 Free Software Foundation, Inc.
+;;;; Copyright (C) 2002, 2003, 2004, 2006 Free Software Foundation, Inc.
 ;;;;
 ;;;; GNU Mailutils is free software; you can redistribute it and/or modify
 ;;;; it under the terms of the GNU General Public License as published by
@@ -17,9 +17,13 @@
 ;;;; Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 ;;;;
 
-(define-module (mailutils))
+(define-module (mailutils mailutils)
+               #:use-module (ice-9 documentation))
 
-(define mu-libs (list "libmailbox"
+(set! documentation-files (append documentation-files 
+                                  (list "/usr/local/share/mailutils/0.6.95/guile/mailutils/guile-procedures.txt")))
+
+(define mu-libs (list "libmailutils"
 		      "libmuauth"
 		      "libmu_mbox"
 		      "libmu_mh"
@@ -38,10 +42,10 @@
     (dynamic-call "mu_scm_init"
 		  (dynamic-link (string-append
 				 lib-path
-				 "libguile-mailutils-v-0.6.93"))))
+				 "libguile-mailutils-v-0.6.95"))))
    (else
     (load-extension (string-append
-		     lib-path "libguile-mailutils-v-0.6.93") "mu_scm_init"))))
+		     lib-path "libguile-mailutils-v-0.6.95") "mu_scm_init"))))
 
 (export mu-package)
 (export mu-package-string)
@@ -104,7 +108,6 @@
 (export mu-mailbox-get-message)
 (export mu-mailbox-messages-count)
 (export mu-mailbox-expunge)
-(export mu-mailbox-url)
 (export mu-mailbox-append-message)
 
 (export mu-message-create)
@@ -128,6 +131,7 @@
 (export mu-message-get-num-parts)
 (export mu-message-get-part)
 (export mu-message-send)
+(export mu-message-get-uid)
 
 (export mu-mime-create)
 (export mu-mime-multipart?)
@@ -142,6 +146,7 @@
 
 
 (export mu-register-format)
+(export mu-strerror)
 
 (export mu-getpwuid)
 
