@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2003, 
-   2004, 2005 Free Software Foundation, Inc.
+   2004, 2005, 2006 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -41,7 +41,9 @@
 
 #ifdef WITH_PTHREAD
 # ifdef HAVE_PTHREAD_H
-#  define _XOPEN_SOURCE  500
+#  ifndef _XOPEN_SOURCE
+#   define _XOPEN_SOURCE  500
+#  endif
 #  include <pthread.h>
 # endif
 #endif
