@@ -39,6 +39,14 @@ char mh_list_format[] =
 "%<(zero)%17(friendly{from})%>"
 "  %{subject}%<{body}<<%{body}>>%>";
 
+static int
+mu_error_printer (const char *fmt, va_list ap)
+{
+  fprintf (stderr, "%s: ", program_invocation_short_name);
+  vfprintf (stderr, fmt, ap);
+  fprintf (stderr, "\n");
+  return 0;
+}
 
 void
 mh_init ()
@@ -46,6 +54,8 @@ mh_init ()
   /* Register all mailbox and mailer formats */
   mu_register_all_formats ();
 
+  mu_error_set_print (mu_error_printer);
+  
   /* Read user's profile */
   mh_read_profile ();
 }
@@ -68,14 +78,14 @@ mh_read_formfile (char *name, char **pformat)
 
   if (stat (name, &st))
     {
-      mh_error (_("Cannot stat format file %s: %s"), name, strerror (errno));
+      mu_error (_("Cannot stat format file %s: %s"), name, strerror (errno));
       return -1;
     }
   
   fp = fopen (name, "r");
   if (!fp)
     {
-      mh_error (_("Cannot open format file %s: %s"), name, strerror (errno));
+      mu_error (_("Cannot open format file %s: %s"), name, strerror (errno));
       return -1;
     }
 
@@ -110,7 +120,7 @@ mh_read_formfile (char *name, char **pformat)
 void
 mh_err_memory (int fatal)
 {
-  mh_error (_("Not enough memory"));
+  mu_error (_("Not enough memory"));
   if (fatal)
     abort ();
 }
@@ -126,7 +136,7 @@ mh_get_my_name (char *name)
       struct passwd *pw = getpwuid (getuid ());
       if (!pw)
 	{
-	  mh_error (_("Cannot determine my username"));
+	  mu_error (_("Cannot determine my username"));
 	  return;
 	}
       name = pw->pw_name;
@@ -214,6 +224,58 @@ mh_my_email ()
   return my_email;
 }
 
+static int
+make_dir_hier (const char *p, mode_t perm)
+{
+  int rc = 0;
+  char *dir = xstrdup (p);
+  char *q = dir;
+
+  while (!rc && (q = strchr (q + 1, '/')))
+    {
+      *q = 0;
+      if (access (dir, X_OK))
+	{
+	  if (errno != ENOENT)
+	    {
+	      mu_error (_("Cannot create directory %s: error accessing name component %s: %s"),
+			dir, strerror (errno));
+	      rc = 1;
+	    }
+	  else if ((rc = mkdir (dir, perm)))
+	    mu_error (_("Cannot create directory %s: error creating name component %s: %s"),
+		      dir, rc);
+	}
+      *q = '/';
+    }
+  free (dir);
+  return rc;
+}
+
+int
+mh_makedir (char *p)
+{
+  int rc;
+  mode_t save_umask;
+  mode_t perm = 0711;
+  char *pb = mh_global_profile_get ("Folder-Protect", NULL);
+  if (pb)
+    perm = strtoul (pb, NULL, 8);
+
+  save_umask = umask (0);
+
+  if ((rc = make_dir_hier (p, perm)) == 0)
+    {
+      rc = mkdir (p, perm);
+      if (rc)
+	mu_error (_("Cannot create directory %s: %s"),
+		  p, strerror (errno));
+    }
+
+  umask (save_umask);
+  return rc;
+}
+
 int
 mh_check_folder (char *pathname, int confirm)
 {
@@ -230,25 +292,13 @@ mh_check_folder (char *pathname, int confirm)
       if (errno == ENOENT)
 	{
 	  if (!confirm || mh_getyn (_("Create folder \"%s\""), p))
-	    {
-	      int perm = 0711;
-	      char *pb = mh_global_profile_get ("Folder-Protect", NULL);
-	      if (pb)
-		perm = strtoul (pb, NULL, 8);
-	      if (mkdir (p, perm)) 
-		{
-		  mh_error (_("Cannot create directory %s: %s"),
-			    p, strerror (errno));
-		  return 1;
-		}
-	      return 0;
-	    }
+	    return mh_makedir (p);
 	  else
 	    return 1;
 	}
       else
 	{
-	  mh_error (_("Cannot stat %s: %s"), p, strerror (errno));
+	  mu_error (_("Cannot stat %s: %s"), p, strerror (errno));
 	  return 1;
 	}
     }
@@ -339,7 +389,7 @@ mh_audit_open (char *name, mu_mailbox_t mbox)
       asprintf (&p, "%s/%s", mu_folder_directory (), namep);
       if (!p)
 	{
-	  mh_error (_("Not enough memory"));
+	  mu_error (_("Not enough memory"));
 	  exit (1);
 	}
       free (namep);
@@ -349,7 +399,7 @@ mh_audit_open (char *name, mu_mailbox_t mbox)
   fp = fopen (namep, "a");
   if (!fp)
     {
-      mh_error (_("Cannot open audit file %s: %s"), namep, strerror (errno));
+      mu_error (_("Cannot open audit file %s: %s"), namep, strerror (errno));
       free (namep);
       return NULL;
     }
@@ -392,7 +442,7 @@ mh_open_folder (const char *folder, int create)
     
   if (mu_mailbox_create_default (&mbox, name))
     {
-      mh_error (_("Cannot create mailbox %s: %s"),
+      mu_error (_("Cannot create mailbox %s: %s"),
 		name, strerror (errno));
       exit (1);
     }
@@ -402,7 +452,7 @@ mh_open_folder (const char *folder, int create)
   
   if (mu_mailbox_open (mbox, flags))
     {
-      mh_error (_("Cannot open mailbox %s: %s"), name, strerror (errno));
+      mu_error (_("Cannot open mailbox %s: %s"), name, strerror (errno));
       exit (1);
     }
 
@@ -478,7 +528,7 @@ mh_iterate (mu_mailbox_t mbox, mh_msgset_t *msgset,
       num = msgset->list[i];
       if ((rc = mu_mailbox_get_message (mbox, num, &msg)) != 0)
 	{
-	  mh_error (_("Cannot get message %d: %s"), num, mu_strerror (rc));
+	  mu_error (_("Cannot get message %d: %s"), num, mu_strerror (rc));
 	  return 1;
 	}
 
@@ -496,7 +546,7 @@ mh_spawnp (const char *prog, const char *file)
 
   if (mu_argcv_get (prog, "", "#", &argc, &argv))
     {
-      mh_error (_("Cannot split line %s"), prog);
+      mu_error (_("Cannot split line %s"), prog);
       mu_argcv_free (argc, argv);
       return 1;
     }
@@ -534,7 +584,7 @@ mh_file_copy (const char *from, const char *to)
   
   if (stat (from, &st))
     {
-      mh_error ("mh_copy: %s", mu_strerror (errno));
+      mu_error ("mh_copy: %s", mu_strerror (errno));
       return -1;
     }
 
@@ -548,7 +598,7 @@ mh_file_copy (const char *from, const char *to)
   if ((rc = mu_file_stream_create (&in, from, MU_STREAM_READ)) != 0
       || (rc = mu_stream_open (in)))
     {
-      mh_error (_("Cannot open input file `%s': %s"),
+      mu_error (_("Cannot open input file `%s': %s"),
 		from, mu_strerror (rc));
       free (buffer);
       return 1;
@@ -557,7 +607,7 @@ mh_file_copy (const char *from, const char *to)
   if ((rc = mu_file_stream_create (&out, to, MU_STREAM_RDWR|MU_STREAM_CREAT)) != 0
       || (rc = mu_stream_open (out)))
     {
-      mh_error (_("Cannot open output file `%s': %s"),
+      mu_error (_("Cannot open output file `%s': %s"),
 		to, mu_strerror (rc));
       free (buffer);
       mu_stream_close (in);
@@ -571,7 +621,7 @@ mh_file_copy (const char *from, const char *to)
     {
       if ((rc = mu_stream_sequential_write (out, buffer, rdsize)) != 0)
 	{
-	  mh_error (_("Write error on `%s': %s"),
+	  mu_error (_("Write error on `%s': %s"),
 		    to, mu_strerror (rc));
 	  break;
 	}
@@ -588,32 +638,29 @@ mh_file_copy (const char *from, const char *to)
   return rc;
 }
 
-mu_message_t
-mh_file_to_message (const char *folder, char *file_name)
+static mu_message_t
+_file_to_message (char *file_name)
 {
   struct stat st;
   int rc;
   mu_stream_t instream;
-  
-  if (folder)
-    file_name = mh_expand_name (folder, file_name, 0);
-  
+
   if (stat (file_name, &st) < 0)
     {
-      mh_error (_("Cannot stat file %s: %s"), file_name, strerror (errno));
+      mu_error (_("Cannot stat file %s: %s"), file_name, strerror (errno));
       return NULL;
     }
   
   if ((rc = mu_file_stream_create (&instream, file_name, MU_STREAM_READ)))
     {
-      mh_error (_("Cannot create input stream (file %s): %s"),
+      mu_error (_("Cannot create input stream (file %s): %s"),
 		file_name, mu_strerror (rc));
       return NULL;
     }
   
   if ((rc = mu_stream_open (instream)))
     {
-      mh_error (_("Cannot open input stream (file %s): %s"),
+      mu_error (_("Cannot open input stream (file %s): %s"),
 		file_name, mu_strerror (rc));
       mu_stream_destroy (&instream, mu_stream_get_owner (instream));
       return NULL;
@@ -622,6 +669,24 @@ mh_file_to_message (const char *folder, char *file_name)
   return mh_stream_to_message (instream);
 }
 
+mu_message_t
+mh_file_to_message (const char *folder, char *file_name)
+{
+  mu_message_t msg;
+  char *tmp_name = NULL;
+  
+  if (folder)
+    {
+      tmp_name = mh_expand_name (folder, file_name, 0);
+      msg = _file_to_message (tmp_name);
+      free (tmp_name);
+    }
+  else
+    msg = _file_to_message (file_name);
+  
+  return msg;
+}
+
 void
 mh_install_help (char *mhdir)
 {
@@ -725,7 +790,7 @@ mh_install (char *name, int automode)
 	}
       else
 	{
-	  mh_error(_("Cannot stat %s: %s"), name, mu_strerror (errno));
+	  mu_error(_("Cannot stat %s: %s"), name, mu_strerror (errno));
 	  exit (1);
 	}
     }
@@ -811,7 +876,7 @@ mh_set_reply_regex (const char *str)
   char *err;
   int rc = mu_unre_set_regex (str, 0, &err);
   if (rc)
-    mh_error ("reply_regex: %s%s%s", mu_strerror (rc),
+    mu_error ("reply_regex: %s%s%s", mu_strerror (rc),
 	      err ? ": " : "",
 	      err ? err : "");
 }
@@ -979,4 +1044,3 @@ mh_draft_message (const char *name, const char *msgspec, char **pname)
   mu_mailbox_destroy (&mbox);
   return rc;
 }
-
