@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005, 2006 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+   MA 02110-1301 USA */
 
 #include <mh.h>
 
@@ -22,22 +23,22 @@ struct recipient {
   int isbcc;
 };
 
-static list_t local_rcp;   /* Local recipients */
-static list_t network_rcp; /* Network recipients */
+static mu_list_t local_rcp;   /* Local recipients */
+static mu_list_t network_rcp; /* Network recipients */
 
 static void
-addrcp (list_t *list, char *addr, int isbcc)
+addrcp (mu_list_t *list, char *addr, int isbcc)
 {
   int rc;
   struct recipient *p = xmalloc (sizeof (*p));
   p->addr = addr;
   p->isbcc = isbcc;
-  if (!*list && (rc = list_create (list)))
+  if (!*list && (rc = mu_list_create (list)))
     {
-      mh_error (_("Cannot create list: %s"), mu_strerror (rc));
+      mu_error (_("Cannot create list: %s"), mu_strerror (rc));
       exit (1);
     }
-  list_append (*list, p);
+  mu_list_append (*list, p);
 }
 
 static int
@@ -51,15 +52,17 @@ ismydomain (char *p)
 }
 
 int
-mh_alias_expand (char *str, address_t *paddr, int *incl)
+mh_alias_expand (char *str, mu_address_t *paddr, int *incl)
 {
   size_t argc;
   char **argv;
   size_t i;
   char *buf;
-  address_t exaddr = NULL;
-  
-  argcv_get (str, ",", NULL, &argc, &argv);
+  mu_address_t exaddr = NULL;
+
+  if (incl)
+    *incl = 0;
+  mu_argcv_get (str, ",", NULL, &argc, &argv);
   for (i = 0; i < argc;)
     {
       if (i + 1 == argc)
@@ -101,16 +104,16 @@ mh_alias_expand (char *str, address_t *paddr, int *incl)
   if (argc)
     {
       int status;
-      argcv_string (argc, argv, &buf);
-      if (status = address_create (paddr, buf))
-	mh_error (_("Bad address `%s': %s"), buf, mu_strerror (status));
+      mu_argcv_string (argc, argv, &buf);
+      if (status = mu_address_create (paddr, buf))
+	mu_error (_("Bad address `%s': %s"), buf, mu_strerror (status));
       free (buf);
     }
 
-  argcv_free (argc, argv);
+  mu_argcv_free (argc, argv);
   
-  address_union (paddr, exaddr);
-  address_destroy (&exaddr);
+  mu_address_union (paddr, exaddr);
+  mu_address_destroy (&exaddr);
   return 0;
 }
 
@@ -118,7 +121,7 @@ mh_alias_expand (char *str, address_t *paddr, int *incl)
 static void
 scan_addrs (char *str, int isbcc)
 {
-  address_t addr = NULL;
+  mu_address_t addr = NULL;
   size_t i, count;
   char *buf;
   int rc;
@@ -128,17 +131,17 @@ scan_addrs (char *str, int isbcc)
 
   mh_alias_expand (str, &addr, NULL);
     
-  if (addr == NULL || address_get_count (addr, &count))
+  if (addr == NULL || mu_address_get_count (addr, &count))
     return;
     
   for (i = 1; i <= count; i++)
     {
       char *p;
 
-      rc = address_aget_email (addr, i, &buf);
+      rc = mu_address_aget_email (addr, i, &buf);
       if (rc)
 	{
-	  mh_error ("address_aget_email: %s", mu_strerror (rc));
+	  mu_error ("mu_address_aget_email: %s", mu_strerror (rc));
 	  continue;
 	}
 
@@ -149,7 +152,7 @@ scan_addrs (char *str, int isbcc)
       else
 	addrcp (&network_rcp, buf, isbcc);
     }
-  address_destroy (&addr);
+  mu_address_destroy (&addr);
   free (str); /* FIXME: This will disappear. Note comment to
 		 mh_context_get_value! */
 }
@@ -164,12 +167,12 @@ _destroy_recipient (void *item, void *unused_data)
 }
 
 static void
-destroy_addrs (list_t *list)
+destroy_addrs (mu_list_t *list)
 {
   if (!*list)
     return;
-  list_do (*list, _destroy_recipient, NULL);
-  list_destroy (list);
+  mu_list_do (*list, _destroy_recipient, NULL);
+  mu_list_destroy (list);
 }
 
 /* Print an email in more readable form: localpart + "at" + domain */
@@ -224,7 +227,7 @@ mh_whom (char *filename, int check)
   ctx = mh_context_create (filename, 1);
   if (mh_context_read (ctx))
     {
-      mh_error (_("Malformed message"));
+      mu_error (_("Malformed message"));
       rc = -1;
     }
   else
@@ -238,18 +241,18 @@ mh_whom (char *filename, int check)
       if (local_rcp)
 	{
 	  printf ("  %s\n", _("-- Local Recipients --"));
-	  list_do (local_rcp, _print_local_recipient, &count);
+	  mu_list_do (local_rcp, _print_local_recipient, &count);
 	}
 
       if (network_rcp)
 	{
 	  printf ("  %s\n", _("-- Network Recipients --"));
-	  list_do (network_rcp, _print_recipient, &count);
+	  mu_list_do (network_rcp, _print_recipient, &count);
 	}
 
       if (count == 0)
 	{
-	  mh_error(_("No recipients"));
+	  mu_error(_("No recipients"));
 	  rc = -1;
 	}
     }
