@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005, 2006 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -519,7 +519,7 @@ msg_part_format_stk (struct obstack *stk, msg_part_t p)
   for (i = 1; i <= p->level; i++)
     {
       int len;
-      char *buf;
+      const char *buf;
   
       if (i > 1)
 	obstack_1grow (stk, '.');
@@ -549,7 +549,7 @@ msg_part_parse (char *str)
 	  str = endp;
 	  break;
 	default:
-	  mh_error (_("Malformed part specification (near %s)"), endp);
+	  mu_error (_("Malformed part specification (near %s)"), endp);
 	  exit (1);
 	}
       msg_part_incr (p);
@@ -793,7 +793,6 @@ mhn_store_command (mu_message_t msg, msg_part_t part, char *name)
   char *typestr, *type, *subtype, *typeargs;
   struct obstack stk;
   mu_header_t hdr;
-  char *buf;
   
   mu_message_get_header (msg, &hdr);
   _get_content_type (hdr, &typestr, &typeargs);
@@ -823,7 +822,7 @@ mhn_store_command (mu_message_t msg, msg_part_t part, char *name)
 		obstack_grow (&stk, name, strlen (name));
 	      else
 		{
-                  buf = mu_umaxtostr (0, msg_part_subpart (part, 0));
+                  const char *buf = mu_umaxtostr (0, msg_part_subpart (part, 0));
 		  obstack_grow (&stk, buf, strlen (buf));
 		}
 	      break;
@@ -881,6 +880,32 @@ mhn_store_command (mu_message_t msg, msg_part_t part, char *name)
 
 /* ************************* Auxiliary functions ************************** */
 
+static void
+split_args (char *argstr, int *pargc, char ***pargv)
+{
+  int argc;
+  char **argv;
+
+  if (mu_argcv_get (argstr, ";", NULL, &argc, &argv) == 0)
+    {
+      int i, j;
+      
+      for (i = j = 0; i < argc; i++)
+	{
+	  if (argv[i][0] != ';')
+	    argv[j++] = argv[i];
+	}
+      argv[j] = NULL;
+      *pargc = j;
+      *pargv = argv;
+    }
+  else
+    {
+      *pargc = 0;
+      *pargv = NULL;
+    }
+}
+
 int
 _message_is_external_body (mu_message_t msg, char ***env)
 {
@@ -895,22 +920,8 @@ _message_is_external_body (mu_message_t msg, char ***env)
   rc = subtype && strcmp (subtype, "external-body") == 0;
   if (rc && env)
     {
-      int argc;
-      char **argv;
-      if (mu_argcv_get (argstr, ";", NULL, &argc, &argv) == 0)
-	{
-	  int i, j;
-	  
-	  for (i = j =  0; i < argc; i++)
-	    {
-	      if (argv[i][0] != ';')
-		argv[j++] = argv[i];
-	    }
-	  argv[j] = NULL;
-	  *env = argv;
-	}
-      else
-	*env = NULL;
+      int c;
+      split_args (argstr, &c, env);
     }
 
   free (typestr);
@@ -1251,7 +1262,7 @@ show_internal (mu_message_t msg, msg_part_t part, char *encoding, mu_stream_t ou
 
   if ((rc = mu_message_get_body (msg, &body)))
     {
-      mh_error (_("%s: cannot get message body: %s"),
+      mu_error (_("%s: cannot get message body: %s"),
 		mu_umaxtostr (0, msg_part_subpart (part, 0)),
 		mu_strerror (rc));
       return 0;
@@ -1273,14 +1284,14 @@ mhn_exec (mu_stream_t *str, char *cmd, int flags)
   int rc = mu_prog_stream_create (str, cmd, MU_STREAM_WRITE);
   if (rc)
     {
-      mh_error (_("Cannot create proc stream (command %s): %s"),
+      mu_error (_("Cannot create proc stream (command %s): %s"),
 		cmd, mu_strerror (rc));
     }
   else
     {
       rc = mu_stream_open (*str);
       if (rc)
-	mh_error (_("Cannot open proc stream (command %s): %s"),
+	mu_error (_("Cannot open proc stream (command %s): %s"),
 		  cmd, mu_strerror (rc));
     }
   return rc;
@@ -1321,14 +1332,14 @@ mhn_run_command (mu_message_t msg, msg_part_t part,
       
       if (mu_argcv_get (cmd, "", "#", &argc, &argv))
 	{
-	  mh_error (_("Cannot parse command line `%s'"), cmd);
+	  mu_error (_("Cannot parse command line `%s'"), cmd);
 	  return ENOSYS;
 	}
 
       rc = mu_file_stream_create (&tmp, tempfile, MU_STREAM_RDWR);
       if (rc)
 	{
-	  mh_error (_("Cannot create temporary stream (file %s): %s"),
+	  mu_error (_("Cannot create temporary stream (file %s): %s"),
 		    tempfile, mu_strerror (rc));
 	  mu_argcv_free (argc, argv);
 	  return rc;
@@ -1336,7 +1347,7 @@ mhn_run_command (mu_message_t msg, msg_part_t part,
       rc = mu_stream_open (tmp);
       if (rc)
 	{
-	  mh_error (_("Cannot open temporary stream (file %s): %s"),
+	  mu_error (_("Cannot open temporary stream (file %s): %s"),
 		    tempfile, mu_strerror (rc));
 	  mu_stream_destroy (&tmp, mu_stream_get_owner (tmp));
 	  mu_argcv_free (argc, argv);
@@ -1371,7 +1382,7 @@ show_handler (mu_message_t msg, msg_part_t part, char *type, char *encoding,
     return 0;
 
   mu_stream_get_transport (out, &trans);
-  fd = trans;
+  fd = (int) trans; /* FIXME */
 
   if (mode_options & OPT_PAUSE)
     flags |= MHN_CONFIRM;
@@ -1381,7 +1392,7 @@ show_handler (mu_message_t msg, msg_part_t part, char *type, char *encoding,
   if (flags & MHN_LISTING)
     {
       char *str;
-      char *p;
+      const char *p;
       size_t size = 0;
 
       str = _("part ");
@@ -1462,13 +1473,13 @@ mhn_show ()
   rc = mu_stdio_stream_create (&ostr, stdout, 0);
   if (rc)
     {
-      mh_error (_("Cannot create output stream: %s"), mu_strerror (rc));
+      mu_error (_("Cannot create output stream: %s"), mu_strerror (rc));
       exit (1);
     }
   rc = mu_stream_open (ostr);
   if (rc)
     {
-      mh_error (_("Cannot open output stream: %s"), mu_strerror (rc));
+      mu_error (_("Cannot open output stream: %s"), mu_strerror (rc));
       exit (1);
     }
   
@@ -1677,7 +1688,7 @@ store_handler (mu_message_t msg, msg_part_t part, char *type, char *encoding,
   rc = mu_file_stream_create (&out, name, MU_STREAM_WRITE|MU_STREAM_CREAT);
   if (rc)
     {
-      mh_error (_("Cannot create output stream (file %s): %s"),
+      mu_error (_("Cannot create output stream (file %s): %s"),
 		name, mu_strerror (rc));
       free (name);
       return rc;
@@ -1685,7 +1696,7 @@ store_handler (mu_message_t msg, msg_part_t part, char *type, char *encoding,
   rc = mu_stream_open (out);
   if (rc)
     {
-      mh_error (_("Cannot open output stream (file %s): %s"),
+      mu_error (_("Cannot open output stream (file %s): %s"),
 		name, mu_strerror (rc));
       free (name);
       mu_stream_destroy (&out, mu_stream_get_owner (out));
@@ -1799,7 +1810,7 @@ parse_brace (char **pval, char **cmd, int c, struct compose_env *env)
   
   if (!sp)
     {
-      mh_error (_("%s:%lu: missing %c"),
+      mu_error (_("%s:%lu: missing %c"),
 		input_file,
 		(unsigned long) mhn_error_loc (env),
 		c);
@@ -1834,7 +1845,7 @@ parse_content_type (struct compose_env *env,
 	case '(':
 	  if (comment)
 	    {
-	      mh_error (_("%s:%lu: comment redefined"),
+	      mu_error (_("%s:%lu: comment redefined"),
 			input_file,
 			(unsigned long) mhn_error_loc (env));
 	      status = 1;
@@ -1846,7 +1857,7 @@ parse_content_type (struct compose_env *env,
 	case '[':
 	  if (!descr)
 	    {
-	      mh_error (_("%s:%lu: syntax error"),
+	      mu_error (_("%s:%lu: syntax error"),
 			input_file,
 			(unsigned long) mhn_error_loc (env));
 	      status = 1;
@@ -1855,7 +1866,7 @@ parse_content_type (struct compose_env *env,
 	    
 	  if (*descr)
 	    {
-	      mh_error (_("%s:%lu: description redefined"),
+	      mu_error (_("%s:%lu: description redefined"),
 			input_file,
 			(unsigned long) mhn_error_loc (env));
 	      status = 1;
@@ -1867,7 +1878,7 @@ parse_content_type (struct compose_env *env,
 	case '<':
 	  if (*id)
 	    {
-	      mh_error (_("%s:%lu: content id redefined"),
+	      mu_error (_("%s:%lu: content id redefined"),
 			input_file,
 			(unsigned long) mhn_error_loc (env));
 	      status = 1;
@@ -1886,7 +1897,7 @@ parse_content_type (struct compose_env *env,
 	  skipws (rest);
 	  if (*rest != '=')
 	    {
-	      mh_error (_("%s:%lu: syntax error"),
+	      mu_error (_("%s:%lu: syntax error"),
 			input_file,
 			(unsigned long) mhn_error_loc (env));
 	      status = 1;
@@ -1949,7 +1960,7 @@ parse_type_command (char **pcmd, struct compose_env *env, mu_header_t hdr)
   
   if (!subtype)
     {
-      mh_error (_("%s:%lu: missing subtype"),
+      mu_error (_("%s:%lu: missing subtype"),
 		input_file,
 		(unsigned long) mhn_error_loc (env));
       return 1;
@@ -1981,6 +1992,53 @@ parse_type_command (char **pcmd, struct compose_env *env, mu_header_t hdr)
   return status;
 }
 
+void
+copy_header (mu_message_t msg, mu_header_t out)
+{
+  size_t i, count;
+  mu_header_t hdr;
+
+  mu_message_get_header (msg, &hdr);
+  mu_header_get_field_count (hdr, &count);
+  for (i = 1; i <= count; i++)
+    {
+      char *name, *value;
+
+      if (mu_header_aget_field_name (hdr, i, &name))
+        continue;
+
+      if (mu_header_aget_field_value (hdr, i, &value))
+	{
+          free (name);
+          continue;
+        }
+
+      mu_header_set_value (out, name, value, 0);
+      free (name);
+      free (value);
+    }
+}
+
+void
+copy_header_to_stream (mu_message_t msg, mu_stream_t stream)
+{
+  mu_header_t hdr;
+  mu_stream_t in;
+  char *buf = NULL;
+  size_t bufsize = 0, n = 0;
+  
+  mu_message_get_header (msg, &hdr);
+  mu_header_get_stream (hdr, &in);
+  mu_stream_seek (in, 0, SEEK_SET);
+  while (stream_getline (in, &buf, &bufsize, &n) == 0 && n > 0)
+    {
+      if (n == 1 && buf[0] == '\n')
+	break;
+      mu_stream_sequential_write (stream, buf, n);
+    }
+  free (buf);
+}
+
 void
 finish_msg (struct compose_env *env, mu_message_t *msg)
 {
@@ -2015,8 +2073,10 @@ finish_text_msg (struct compose_env *env, mu_message_t *msg, int ascii)
       
       mu_message_create (&newmsg, NULL);
       mu_message_get_header (newmsg, &hdr);
+      mu_header_get_stream (hdr, &output);
+      copy_header (*msg, hdr);
       mu_header_set_value (hdr, MU_HEADER_CONTENT_TRANSFER_ENCODING,
-			"quoted-printable", 0);
+			   "quoted-printable", 0);
       
       mu_message_get_body (newmsg, &body);
       mu_body_get_stream (body, &output);
@@ -2056,7 +2116,7 @@ edit_extern (char *cmd, struct compose_env *env, mu_message_t *msg, int level)
   
   if ((rc = mu_header_create (&hdr2, NULL, 0, NULL)) != 0)
     {
-      mh_error (_("Cannot create header: %s"),
+      mu_error (_("Cannot create header: %s"),
 		mu_strerror (rc));
       return 1;
     }
@@ -2115,7 +2175,7 @@ edit_forw (char *cmd, struct compose_env *env, mu_message_t *pmsg, int level)
 	case '[':
 	  if (descr)
 	    {
-	      mh_error (_("%s:%lu: description redefined"),
+	      mu_error (_("%s:%lu: description redefined"),
 			input_file,
 			(unsigned long) mhn_error_loc (env));
 	      status = 1;
@@ -2127,7 +2187,7 @@ edit_forw (char *cmd, struct compose_env *env, mu_message_t *pmsg, int level)
 	case '<':
 	  if (id)
 	    {
-	      mh_error (_("%s:%lu: content id redefined"),
+	      mu_error (_("%s:%lu: content id redefined"),
 			input_file,
 			(unsigned long) mhn_error_loc (env));
 	      status = 1;
@@ -2148,7 +2208,7 @@ edit_forw (char *cmd, struct compose_env *env, mu_message_t *pmsg, int level)
 
   if (mu_argcv_get (cmd, "\n", NULL, &argc, &argv))
     {
-      mh_error (_("%s:%lu: syntax error"),
+      mu_error (_("%s:%lu: syntax error"),
 		input_file,
 		(unsigned long) mhn_error_loc (env));
       return 1;
@@ -2162,7 +2222,7 @@ edit_forw (char *cmd, struct compose_env *env, mu_message_t *pmsg, int level)
       mu_message_t input_msg;
       if (mh_get_message (mbox, i, &input_msg) == 0)
 	{
-	  mh_error (_("%s:%lu: no such message: %lu"),
+	  mu_error (_("%s:%lu: no such message: %lu"),
 		    input_file,
 		    (unsigned long) mhn_error_loc (env),
 		    (unsigned long)i);
@@ -2253,7 +2313,7 @@ edit_mime (char *cmd, struct compose_env *env, mu_message_t *msg, int level)
     }
   else if (p == cmd)
     {
-      mh_error (_("%s:%lu: missing filename"),
+      mu_error (_("%s:%lu: missing filename"),
 		input_file,
 		(unsigned long) mhn_error_loc (env));
       finish_msg (env, msg);
@@ -2264,14 +2324,14 @@ edit_mime (char *cmd, struct compose_env *env, mu_message_t *msg, int level)
       rc = mu_file_stream_create (&in, cmd, MU_STREAM_READ);
       if (rc)
 	{
-	  mh_error (_("Cannot create input stream (file %s): %s"),
+	  mu_error (_("Cannot create input stream (file %s): %s"),
 		    cmd, mu_strerror (rc));
 	  return rc;
 	}
       rc = mu_stream_open (in);
       if (rc)
 	{
-	  mh_error (_("Cannot open input stream (file %s): %s"),
+	  mu_error (_("Cannot open input stream (file %s): %s"),
 		    cmd, mu_strerror (rc));
 	  mu_stream_destroy (&in, mu_stream_get_owner (in));
 	  return rc;
@@ -2309,7 +2369,7 @@ edit_mime (char *cmd, struct compose_env *env, mu_message_t *msg, int level)
   rc = mu_stream_open (fstr);
   if (rc)
     {
-      mh_error (_("Cannot open filter stream: %s"),
+      mu_error (_("Cannot open filter stream: %s"),
 		mu_strerror (rc));
       return rc;
     }
@@ -2408,7 +2468,10 @@ mhn_edit (struct compose_env *env, int level)
 	      if (tok[1] == 0)
 		/* starts a new message */;
 	      else if (tok[1] == '<')
-		status = edit_modify (tok+2, env, &msg);
+		{
+		  *sp = c;
+		  status = edit_modify (tok+2, env, &msg);
+		}
 	      else if (tok[1] == '@')
 		{
 		  *sp = c;
@@ -2442,7 +2505,7 @@ mhn_edit (struct compose_env *env, int level)
 		{
 		  if (level == 0)
 		    {
-		      mh_error (_("%s:%lu: unmatched #end"),
+		      mu_error (_("%s:%lu: unmatched #end"),
 				input_file,
 				(unsigned long) mhn_error_loc (env));
 		      status = 1;
@@ -2478,24 +2541,134 @@ mhn_edit (struct compose_env *env, int level)
   return status;
 }
 
-void
-copy_header (mu_message_t msg, mu_stream_t stream)
+int
+parse_header_directive (char *val, char **encoding, char **charset, char **subject)
 {
-  mu_header_t hdr;
-  mu_stream_t in;
-  char *buf = NULL;
-  size_t bufsize = 0, n = 0;
+  char *p;
+
+  /* Provide default values */
+  *encoding = NULL;
+  *charset = NULL;
+  *subject = val;
+
+  if (*val != '#')
+    return 1;
+  val++;
+  
+  switch (*val)
+    {
+    case '#':
+      *subject = val;
+      return 1;
+
+    case '<':
+      for (p = val; *p; p++)
+	if (*p == '>')
+	  {
+	    int i, argc;
+	    char **argv;
+	    
+	    *subject = p + 1;
+	    *p = 0;
+	    split_args (val + 1, &argc, &argv);
+	    for (i = 0; i < argc; i++)
+	      {
+		if (strlen (argv[i]) > 8
+		    && memcmp (argv[i], "charset=", 8) == 0)
+		  {
+		    free (*charset);
+		    *charset = strdup (argv[i] + 8);
+		  }
+		else if (strlen (argv[i]) > 9
+			 && memcmp (argv[i], "encoding=", 9) == 0)
+		  {
+		    free (*encoding);
+		    *encoding = strdup (argv[i] + 9);
+		  }
+	      }
+	    mu_argcv_free (argc, argv);
+	    return 0;
+	  }
+      break;
+    }
+  return 1;
+}
+
+
+void
+mhn_header (mu_message_t msg, mu_message_t omsg)
+{
+  mu_header_t hdr = NULL;
+  char *val;
   
   mu_message_get_header (msg, &hdr);
-  mu_header_get_stream (hdr, &in);
-  mu_stream_seek (in, 0, SEEK_SET);
-  while (stream_getline (in, &buf, &bufsize, &n) == 0 && n > 0)
+  if (mu_header_aget_value (hdr, MU_HEADER_SUBJECT, &val) == 0)
     {
-      if (n == 1 && buf[0] == '\n')
-	break;
-      mu_stream_sequential_write (stream, buf, n);
+      char *subject, *encoding, *charset;
+      
+      if (parse_header_directive (val, &encoding, &charset, &subject))
+	{
+	  if (has_nonascii (val, strlen (val)))
+	    {
+	      int ismime = 0;
+	      mu_message_t part = NULL;
+	      
+	      mu_message_is_multipart (omsg, &ismime);
+	      if (ismime)
+		mu_message_get_part (omsg, 1, &part);
+	      else
+		part = omsg;
+	      if (part);
+		{
+		  mu_header_t parthdr = NULL;
+		  char *typestr, *typeargs;
+		  
+		  mu_message_get_header (part, &parthdr);
+		  _get_content_type (parthdr, &typestr, &typeargs);
+		  _get_content_encoding (parthdr, &encoding);
+		  if (typeargs)
+		    {
+		      int i, argc;
+		      char **argv;
+		      
+		      split_args (typeargs, &argc, &argv);
+		      for (i = 0; i < argc; i++)
+			if (strlen (argv[i]) > 8
+			    && memcmp (argv[i], "charset=", 8) == 0)
+			  {
+			    charset = strdup (argv[i]+8);
+			    break;
+			  }
+		      mu_argcv_free (argc, argv);
+		    }
+		  free (typestr);
+		}
+	    }
+	}
+
+      if (charset)
+	{
+	  char *p;
+	  int rc;
+	  
+	  if (!encoding || strcmp (encoding, "7bit") == 0)
+	    {
+	      free (encoding);
+	      encoding = strdup ("base64");
+	    }
+ 	  rc = mu_rfc2047_encode (charset, encoding, subject, &p);
+	  if (rc)
+	    mu_error (_("cannot encode subject using %s, %s: %s"),
+		      charset, encoding, mu_strerror (rc));
+	  else
+	    {
+	      mu_header_set_value (hdr, MU_HEADER_SUBJECT, p, 1);
+	      free (p);
+	    }
+	}
+      free (charset);
+      free (encoding);
     }
-  free (buf);
 }
 
 int
@@ -2518,12 +2691,13 @@ mhn_compose ()
   env.mime = mime;
   env.input = stream;
   env.subpart = 0;
+  env.line = 0;
   rc = mhn_edit (&env, 0);
   if (rc)
     return rc;
 
   mu_mime_get_message (mime, &msg);
-
+  
   p = strrchr (input_file, '/');
   /* Prepare file names */
   if (p)
@@ -2544,7 +2718,7 @@ mhn_compose ()
   rc = mu_file_stream_create (&stream, name, MU_STREAM_RDWR|MU_STREAM_CREAT);
   if (rc)
     {
-      mh_error (_("Cannot create output stream (file %s): %s"),
+      mu_error (_("Cannot create output stream (file %s): %s"),
 		name, mu_strerror (rc));
       free (name);
       return rc;
@@ -2552,14 +2726,15 @@ mhn_compose ()
   rc = mu_stream_open (stream);
   if (rc)
     {
-      mh_error (_("Cannot open output stream (file %s): %s"),
+      mu_error (_("Cannot open output stream (file %s): %s"),
 		name, mu_strerror (rc));
       free (name);
       mu_stream_destroy (&stream, mu_stream_get_owner (stream));
       return rc;
     }
-  
-  copy_header (message, stream);
+
+  mhn_header (message, msg);
+  copy_header_to_stream (message, stream);
   mu_message_get_stream (msg, &in);
   cat_message (stream, in);
   mu_stream_destroy (&stream, mu_stream_get_owner (stream));
@@ -2598,7 +2773,7 @@ main (int argc, char **argv)
     {
       if (argc)
 	{
-	  mh_error (_("extra arguments"));
+	  mu_error (_("extra arguments"));
 	  return 1;
 	}
       message = mh_file_to_message (mu_folder_directory (), input_file);
@@ -2609,7 +2784,7 @@ main (int argc, char **argv)
     {
       if (argc > 1)
 	{
-	  mh_error (_("extra arguments"));
+	  mu_error (_("extra arguments"));
 	  return 1;
 	}
       input_file = argc == 1 ? argv[0] : "draft";
