@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2005, 2006 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+   MA 02110-1301 USA */
 
 /* MH rmf command */
 
@@ -103,11 +104,11 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
 static char *
 current_folder_path ()
 {
-  mailbox_t mbox = mh_open_folder (mh_current_folder (), 0);
-  url_t url;
+  mu_mailbox_t mbox = mh_open_folder (mh_current_folder (), 0);
+  mu_url_t url;
   char *p;
-  mailbox_get_url (mbox, &url);
-  p = (char*) url_to_string (url);
+  mu_mailbox_get_url (mbox, &url);
+  p = (char*) mu_url_to_string (url);
   if (strncmp (p, "mh:", 3) == 0)
     p += 3;
   return p;
@@ -124,7 +125,7 @@ rmf (const char *name)
 
   if (!dir)
     {
-      mh_error (_("Cannot scan folder %s: %s"), name, strerror (errno));
+      mu_error (_("Cannot scan folder %s: %s"), name, strerror (errno));
       return 1;
     }
 
@@ -143,7 +144,7 @@ rmf (const char *name)
       asprintf (&p, "%s/%s", name, entry->d_name);
       if (stat (p, &st) < 0)
 	{
-	  mh_error (_("Cannot stat %s: %s"), p, strerror (errno));
+	  mu_error (_("Cannot stat %s: %s"), p, strerror (errno));
 	}
       else if (S_ISDIR (st.st_mode))
 	{
@@ -160,7 +161,7 @@ rmf (const char *name)
 	{
 	  if (unlink (p))
 	    {
-	      mh_error (_("Cannot unlink %s: %s"), p, strerror (errno));
+	      mu_error (_("Cannot unlink %s: %s"), p, strerror (errno));
 	      failures++;
 	    }
 	}
@@ -198,12 +199,16 @@ main (int argc, char **argv)
   mu_argp_init (program_version, NULL);
   mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, NULL);
-  if (!explicit_folder)
-    interactive = 1;
 
   cur_folder_path = current_folder_path ();
 
-  name = mh_expand_name (NULL, folder_name, 0);
+  if (!explicit_folder)
+    {
+      interactive = 1;
+      name = cur_folder_path;
+    }
+  else
+    name = mh_expand_name (NULL, folder_name, 0);
   rmf (name);
   return 0;
 }
